import os
from shutil import which


class Redis(object):
    def configure(self):
        if which('redis-cli') is None:
            os.system('sudo apt install -y redis-server')

    def start(self):
        os.system('sudo systemctl start redis.service')

    def stop(self):
        os.system('sudo systemctl stop redis.service')

    def restart(self):
        os.system('sudo systemctl restart redis.service')

    def enable(self,):
        os.system('sudo systemctl enable redis.service')

    def disable(self):
        os.system('sudo systemctl disable redis.service')




import os
import re
import subprocess
import json
from packaging import version as packaging_version


class Magento2(object):
    def __init__(self):
        self.site = None
        self.config = None
        self.db = None
        self._version = None
        self.versions = ['2.4.3', '2.4.2-p2', '2.4.2-p1', '2.4.2', '2.4.1-p1', '2.4.1', '2.4.0-p1', '2.4.0',
                         '2.3.7-p1', '2.3.7', '2.3.6-p1', '2.3.6', '2.3.5-p2', '2.3.5-p1', '2.3.5', '2.3.4-p2',
                         '2.3.4', '2.3.3-p1', '2.3.3', '2.3.2-p2', '2.3.2', '2.3.1', '2.3.0']

    @property
    def version(self):
        if self._version is not None:
            return self._version

        magento2_base_composer = rf'{self.site.site_path()}/vendor/magento/magento2-base/composer.json'

        if not os.path.isfile(magento2_base_composer):
            return None

        with open(magento2_base_composer) as jsonFile:
            json_object = json.load(jsonFile)
            jsonFile.close()

        self._version = json_object['version']

        return self._version

    def magento(self, cmd):
        os.system(f'{self.site.site_path()}/bin/magento {cmd}')

    def twofactorauth(self, user, secret):
        self.magento('config:set twofactorauth/general/force_providers google')
        self.magento('config:set twofactorauth/google/otp_window 60')
        self.magento(f'security:tfa:google:set-secret {user} {secret}')

    def config_set(self, path, value, options=''):
        self.magento(f'config:set {path} "{value}" {options}')

    def install_sample_data(self):
        self.magento('sampledata:deploy')
        self.magento('setup:upgrade')

    def clean_static(self):
        subprocess.check_output(['rm -rf var/view_preprocessed'], shell=True)
        subprocess.check_output(['find pub/static -maxdepth 1 -mindepth 1 ! '
                                 '-name ".htaccess" -exec rm -rf {} +'], shell=True)
        self.magento('cache:clean full_page block_html')

    def dev_config(self):
        self.magento('app:config:import')

        self.clean_token_request_log()

        self.config_set('admin/security/password_lifetime', '0')
        self.config_set('admin/security/password_is_forced', '0')

        self.config_set('admin/url/custom_path', 'admin', '--lock-env')
        self.config_set('admin/url/use_custom', '0', '--lock-env')

        if self.config.get('sites').get(self.site.current_site_id).get('secure'):
            self.config_set('web/secure/use_in_frontend', '1')
            self.config_set('web/secure/use_in_adminhtml', '1')
        else:
            self.config_set('web/secure/use_in_frontend', '0')
            self.config_set('web/secure/use_in_adminhtml', '0')

        self.config_set('admin/security/use_form_key', '0')

        self.config_set('admin/security/session_lifetime', '31536000')

        self.config_set('web/cookie/cookie_domain', '')
        self.config_set('web/cookie/cookie_path', '')

        self.config_set('admin/captcha/enable', '0')
        self.config_set('customer/captcha/enable', '0')

        self.config_set('dev/debug/template_hints_storefront', '1')
        self.config_set('dev/debug/template_hints_storefront_show_with_parameter', '1')

        self.config_set('web/seo/use_rewrites', '1')

        self.config_set('google/analytics/active', '0')

        self.config_set('system/full_page_cache/caching_application', '1')

        self.config_set('sales_email/general/async_sending', '0')
        self.config_set('dev/grid/async_indexing', '0')

        self.magento('deploy:mode:set developer')
        self.magento('cache:disable full_page block_html')

        self.magento('app:config:import')

        if os.path.isfile('./vendor/bin/ece-patches'):
            os.system('php ./vendor/bin/ece-patches apply')

        self.magento('cache:clean')

    def change_base_url(self, src, dist, db_name):
        with self.db.db_connect:
            with self.db.db_connect.cursor() as cur:
                cur.execute(f'SELECT * FROM {db_name}.core_config_data WHERE '
                            f'path like "%base_url" OR path like "%base_link_url"')
                result = cur.fetchall()

                sql_update = f"UPDATE {db_name}.core_config_data SET value = %s WHERE config_id = %s"

                for base_url in result:
                    config_id = base_url['config_id']
                    new_url = base_url['value'].replace(src, dist)

                    if re.search("/secure/", base_url['path']):
                        new_url = new_url.replace('http://', 'https://')
                    else:
                        new_url = new_url.replace('https://', 'http://')

                    cur.execute(sql_update, (new_url, config_id))

                self.db.db_connect.commit()

    def token(self, admin_user="admin"):
        os.system(f'oathtool --base32 --totp $(php {self.config.wt_path}/scripts/magento/taf-secret-token.php '
                  f'{admin_user} {self.site.site_path()})')

    # It does not work in all cases
    def recaptcha(self, status):
        public_key = ''
        private_key = ''
        version = 'v2'

        if packaging_version.parse(self.version.split('-')[0]) >= packaging_version.parse('2.4'):
            version = 'v3'

        with open(f"{self.config.wt_config_path}/config/recaptcha_keys.json") as f:
            recaptcha_keys = json.load(f)

        if status == 1:
            public_key = recaptcha_keys[version]['public_key']
            private_key = recaptcha_keys[version]['private_key']

        if version == 'v3':
            self.config_set('recaptcha_backend/type_recaptcha_v3/public_key', public_key)
            self.config_set('recaptcha_backend/type_recaptcha_v3/private_key', private_key)

            self.config_set('recaptcha_frontend/type_recaptcha_v3/public_key', public_key)
            self.config_set('recaptcha_frontend/type_recaptcha_v3/private_key', private_key)
        else:
            self.config_set('msp_securitysuite_recaptcha/general/public_key', public_key)
            self.config_set('msp_securitysuite_recaptcha/general/private_key', private_key)

            self.config_set('msp_securitysuite_recaptcha/backend/enabled', status)
            self.config_set('msp_securitysuite_recaptcha/frontend/enabled', status)

        self.magento('cache:flush')

    def clean_token_request_log(self):
        with self.db.db_connect:
            with self.db.db_connect.cursor() as cur:
                cur.execute(f'TRUNCATE {self.site.current_site_id}.oauth_token_request_log')

    def install(self, site_id, version, admin_user, admin_password, firstname, lastname, mail, base_url, with_sample_data=False):
        php_version = 7.3
        if packaging_version.parse(version) < packaging_version.parse('2.4'):
            php_version = 7.2

        base_url_secure = base_url.replace('http://', 'https://')
        self.site.php.cli_version(php_version)
        self.site.use(php_version, site_id)

        os.system(f'composer create-project --repository-url=https://repo.magento.com/ '
                  f'magento/project-community-edition={version} .')
        self.db.db_create(site_id)

        cmd = list()
        cmd.append('bin/magento setup:install')
        cmd.append(f'--base-url="{base_url}"')
        cmd.append(f'--base-url-secure="{base_url_secure}"')
        cmd.append('--use-rewrites=1')
        cmd.append('--use-sample-data')

        cmd.append('--db-host=127.0.0.1')
        cmd.append(f'--db-name={site_id}')
        cmd.append('--db-user=root')
        cmd.append('--db-password=root')

        cmd.append('--backend-frontname=admin')
        cmd.append(f'--admin-firstname={firstname}')
        cmd.append(f'--admin-lastname={lastname}')
        cmd.append(f'--admin-email={mail}')
        cmd.append(f'--admin-user={admin_user}')
        cmd.append(f'--admin-password={admin_password}')

        cmd.append('--language=pt_BR')
        cmd.append('--currency=BRL')
        cmd.append('--timezone=America/Sao_Paulo')

        if packaging_version.parse(version) >= packaging_version.parse('2.4'):
            cmd.append('--search-engine="elasticsearch7"')
            cmd.append('--elasticsearch-host="127.0.0.1"')
            cmd.append('--elasticsearch-port=9200')
            cmd.append(f'--elasticsearch-index-prefix="{site_id}"')

        cmd.append('--amqp-host="127.0.0.1"')
        cmd.append('--amqp-port="5672"')
        cmd.append('--amqp-user="guest"')
        cmd.append('--amqp-password="guest"')
        cmd.append('--amqp-virtualhost="/"')

        cmd.append('--session-save="redis"')
        cmd.append('--session-save-redis-host="127.0.0.1"')
        cmd.append('--session-save-redis-port=6379')
        cmd.append('--session-save-redis-timeout=1')

        cmd.append('--cache-backend="redis"')
        cmd.append('--cache-backend-redis-server="127.0.0.1"')
        cmd.append('--cache-backend-redis-port=6379')

        cmd.append('--page-cache="redis"')
        cmd.append('--page-cache-redis-server="127.0.0.1"')
        cmd.append('--page-cache-redis-port=6379')

        cmd = ' '.join(cmd)

        os.system(cmd)
        self.dev_config()

        if packaging_version.parse(version) >= packaging_version.parse('2.4'):
            self.twofactorauth(admin_user, 'MFRGGZDF')

        if with_sample_data:
            self.install_sample_data()



from .http import Http
from .php import Php
from .magento2 import Magento2
from .host import Host
from .config import Config
from .mailhog import MailHog
from .redis import Redis
from .rabbitmq import Rabbitmq
from .site import Site, SiteExistsError, SiteNotExistsError, ProxyExistsError, ProxyNotExistsError
from .db import Db, DbExistsError, DbNotExistsError, SqlFileNotExistsError


db = Db()
config = Config()
redis = Redis()
_host = Host(config)
_php = Php(config)
_http = Http(config)
_magento2 = Magento2()
site = Site(config, _host, _http, _php, db, _magento2)
mailhog = MailHog(config, site)
rabbitmq = Rabbitmq()



import os
from shutil import which


class Rabbitmq(object):
    def configure(self):
        if which('rabbitmq-server') is None:
            os.system('sudo apt install -y rabbitmq-server')
            os.system('sudo rabbitmq-plugins enable rabbitmq_management && sudo rabbitmqctl add_user admin admin '
                      '&& sudo rabbitmqctl set_user_tags admin administrator')

    def start(self):
        os.system('sudo systemctl start rabbitmq-server.service')
        pass

    def stop(self):
        os.system('sudo systemctl stop rabbitmq-server.service')
        pass

    def restart(self):
        os.system('sudo systemctl restart rabbitmq-server.service')
        pass

    def enable(self,):
        os.system('sudo systemctl enable rabbitmq-server.service')
        pass

    def disable(self):
        os.system('sudo systemctl disable rabbitmq-server.service')
        pass



from ruamel import yaml
import pathlib
import os
import getpass
from glob import glob


class Config(object):
    def __init__(self):
        self.file_config = rf'{self.wt_config_path}/config/wt.yml'
        self.config = None
        self.reload()
        self._config = None
        self.public_configs = [
            'php/php_admin_value/auto_prepend_file'
        ]

    def reload(self):
        config_file = self.file_config
        if not os.path.isfile(self.file_config):
            config_file = f'{self.wt_path}/templates/config/wt.yml'
        self.config = yaml.YAML().load(open(config_file).read())

    def get(self, key):
        return self.config.get(key)

    def save(self):
        test = pathlib.Path(self.file_config)
        yaml.YAML().dump(self.config, test)

    def save_test(self):
        test = pathlib.Path(self.file_config.replace('.yml', '-test.yml'))
        yaml.YAML().dump(self.config, test)

    @property
    def certificate_path(self):
        return f'{self.wt_config_path}/certificates'

    @property
    def wt_path(self) -> str:
        return str(pathlib.Path(__file__).parent.parent.resolve())

    @property
    def home_path(self):
        return os.path.expanduser("~")

    @property
    def wt_config_path(self):
        return f'{self.home_path}/.wt'

    @property
    def user(self):
        return getpass.getuser()

    @property
    def current_path(self):
        return pathlib.Path().resolve()

    def _set_config(self, parts, value, config):
        if len(parts) == 1:
            if value == dict():
                value = yaml.CommentedMap(value)
            config.insert(1, parts[0], value)
            return True
        elif len(parts):
            if parts[0] not in config:
                config.insert(1, parts[0], yaml.CommentedMap())
            config = config.get(parts[0])
            return self._set_config(parts[1:], value, config)
        return parts

    def set_config(self, path, value):
        parts = path.rstrip('/').split('/')
        return self._set_config(parts, value, self.config)

    def set_site_config(self, path, value, site_id):
        path = f'sites/{site_id}/{path}'
        return self.set_config(path, value)

    def _get_config(self, parts, config):
        if len(parts) == 1:
            if parts[0] not in config:
                return None
            return config.get(parts[0])
        elif parts[0] in config:
            config = config.get(parts[0])
            if config is None:
                return None
            return self._get_config(parts[1:], config)

    def get_config(self, path):
        parts = path.split('/')
        return self._get_config(parts, self.config)

    def get_site_config(self, path, site_id):
        path = f'sites/{site_id}/{path}'
        return self.get_config(path)

    def _unset_config(self, parts, config):
        try:
            if len(parts) == 2:
                config.get(parts[0]).pop(parts[1], None)
                if config.get(parts[0]) == {}:
                    config.pop(parts[0], None)
                return True
            elif len(parts):
                config = config.get(parts[0])
                return self._unset_config(parts[1:], config)
        except AttributeError:
            return False

    def unset_config(self, path):
        parts = path.rstrip('/').split('/')
        return self._unset_config(parts, self.config)

    def unset_site_config(self, path, site_id):
        path = f'sites/{site_id}/{path}'
        return self.unset_config(path)

    def config_path_complete(self, ctx, args, incomplete):
        return [k for k in self.public_configs if incomplete in k]

    def config_value_complete(self, ctx, args, incomplete):
        values = glob('*.php')
        return [k for k in values if incomplete in k]



import os
import click
import wt_plus.core

from prettytable import PrettyTable

db = click.Group(name='db')


@db.command('ls')
def db_list():
    table = PrettyTable(['Databases'])
    table.align = 'l'

    databases = wt_plus.core.db.db_list()

    for database in databases:
        table.add_row([database])

    click.echo(table)


@db.command('create')
@click.argument('db_name', default=wt_plus.core.site.default_site_id())
def db_create(db_name):
    try:
        wt_plus.core.db.db_create(db_name)
    except wt_plus.core.DbExistsError as e:
        click.echo(e)


@db.command('drop')
@click.argument('db_name', default=wt_plus.core.site.default_site_id(), autocompletion=wt_plus.core.db.databases_complete)
def db_drop(db_name):
    if not wt_plus.core.db.db_exists(db_name):
        click.echo(wt_plus.core.db.MSG_DB_NOT_EXISTS.format(db_name=db_name))
        return

    click.confirm("Remove the '{db_name}' database? 😮".format(db_name=db_name), abort=True)
    wt_plus.core.db.db_drop(db_name)


@db.command('export')
@click.argument('sql_name', default='-')
@click.argument('db_name', default=wt_plus.core.site.default_site_id(), autocompletion=wt_plus.core.db.databases_complete)
@click.option('--sql', default=False, is_flag=True, help='Plain sql to dump')
def db_dump(sql_name, db_name, sql):
    try:
        wt_plus.core.db.db_dump(sql_name, db_name, sql)
    except wt_plus.core.DbNotExistsError as e:
        click.echo(e)


@db.command('reset')
@click.argument('db_dest', default=wt_plus.core.site.default_site_id(), autocompletion=wt_plus.core.db.databases_complete)
def db_reset(db_dest):
    if wt_plus.core.db.db_exists(db_dest):
        click.confirm("Overwrite the '{db_dest}' database? 😮".format(db_dest=db_dest), abort=True)
        wt_plus.core.db.db_drop(db_dest)

    wt_plus.core.db.db_create(db_dest)


@db.command('remove-definer')
@click.argument('sql_path', type=click.Path(exists=True), autocompletion=wt_plus.core.db.sql_file_complete)
def db_remove_definer(sql_path):
    wt_plus.core.db.db_remove_definer(sql_path)


@db.command('import')
@click.argument('sql_path', type=click.Path(exists=True), autocompletion=wt_plus.core.db.sql_file_complete)
@click.argument('db_dest', default=wt_plus.core.site.default_site_id(), autocompletion=wt_plus.core.db.databases_complete)
def db_import(sql_path, db_dest):
    if wt_plus.core.db.db_exists(db_dest):
        click.confirm("Overwrite the '{db_dest}' database? 😮".format(db_dest=db_dest), abort=True)
        wt_plus.core.db.db_drop(db_dest)

    wt_plus.core.db.db_create(db_dest)

    try:
        wt_plus.core.db.db_import(sql_path, db_dest)
    except wt_plus.core.SqlFileNotExistsError as e:
        click.echo(e)


@db.command('clone')
@click.argument('db_src', autocompletion=wt_plus.core.db.databases_complete)
@click.argument('db_dest')
def db_clone(db_src, db_dest):
    click.echo(f"Copying '{db_src}' to '{db_dest}' 📸")

    if not wt_plus.core.db.db_exists(db_src):
        click.echo(wt_plus.core.db.MSG_DB_NOT_EXISTS.format(db_dest=db_src))
        return

    if wt_plus.core.db.db_exists(db_dest):
        click.confirm(f"Overwrite the '{db_dest}' database? 😮", abort=True)
        os.system("mysql -e\"DROP DATABASE {db_dest}\"".format(db_dest=db_dest))

    os.system("mysql -e\"CREATE DATABASE {db_dest}\"".format(db_dest=db_dest))

    cmd = "mysqldump {db_src} | mysql {db_dest}"
    os.system(cmd.format(db_src=db_src, db_dest=db_dest))



from wt_plus.cli.wt import wt
from wt_plus.cli.db import db
from wt_plus.cli.magento2 import magento2
from wt_plus.cli.tools import tools
from wt_plus.cli.services import services

wt.add_command(db)
wt.add_command(magento2)
wt.add_command(tools)
wt.add_command(services)

if __name__ == '__main__':
    wt()



import subprocess
import os
import click
import os.path
import magic
import uuid
import pymysql.cursors
from glob import glob
from datetime import datetime
from configparser import ConfigParser


class DbExistsError(TypeError):
    pass


class DbNotExistsError(TypeError):
    pass


class SqlFileNotExistsError(TypeError):
    pass


class Db:
    DATE_FORMAT = '%Y-%m-%d-%H%M%S'
    MSG_DB_EXISTS = "{db_name} database already exists! 😖"
    MSG_DB_NOT_EXISTS = "Banco '{db_name}' não existe! 😖"

    def __init__(self):
        self._db_connect = None

    def configure(self):
        pass

    @property
    def db_connect(self):
        if self._db_connect is None:
            config = ConfigParser()
            config.read(f"{os.path.expanduser('~')}/.my.cnf")

            host = config.get('client', 'host', fallback='127.0.0.1')
            user = config.get('client', 'user')
            password = config.get('client', 'password')

            self._db_connect = pymysql.connect(host=host,
                                               user=user,
                                               passwd=password,
                                               cursorclass=pymysql.cursors.DictCursor)

        return self._db_connect

    def start(self):
        os.system('sudo systemctl start mysql.service')

    def stop(self):
        os.system('sudo systemctl stop mysql.service')

    def restart(self):
        os.system('sudo systemctl restart mysql.service')

    def enable(self):
        os.system('sudo systemctl enable mysql.service')

    def disable(self):
        os.system('sudo systemctl disable mysql.service')

    def db_list(self):
        databases = list()

        with self.db_connect:
            with self.db_connect.cursor() as cur:
                cur.execute("show databases")
                result = cur.fetchall()
                for database in result:
                    if database['Database'] not in ['mysql', 'information_schema', 'performance_schema', 'phpmyadmin']:
                        databases.append(database['Database'])

        return databases

    def db_sql(self, sql):
        return 'mysql -e "{sql}"'.format(sql=sql)

    def db_exists(self, db_name):
        cmd = "mysql -e \"SELECT if(COUNT(*) = 0, 0, 1) FROM INFORMATION_SCHEMA.SCHEMATA WHERE SCHEMA_NAME = '{db_name}'\" | sed -n 2p | tr -d '\n'"
        output = subprocess.check_output([cmd.format(db_name=db_name)], shell=True)

        return '1' == output.decode('utf-8')

    def db_create(self, db_name):
        if self.db_exists(db_name):
            raise DbExistsError(self.MSG_DB_EXISTS.format(db_name=db_name))

        cmd = "mysql -e \"CREATE DATABASE {db_name}\""
        os.system(cmd.format(db_name=db_name))

    def db_drop(self, db_name):
        if not self.db_exists(db_name):
            raise DbExistsError(self.MSG_DB_NOT_EXISTS.format(db_name=db_name))

        os.system("mysql -e\"DROP DATABASE {db_name}\"".format(db_name=db_name))

    def db_reset(self, db_name):
        self.db_drop(db_name)
        self.db_create(db_name)

    def db_dump(self, file_name, db_name, is_sql):
        if not self.db_exists(db_name):
            raise DbNotExistsError(self.MSG_DB_NOT_EXISTS.format(db_name=db_name))

        now = datetime.now()

        cmd = "mysqldump --column-statistics=0 {db_name} "

        if file_name == '-':
            file_name = "{db_name}-{now}.sql".format(now=now.strftime(self.DATE_FORMAT), db_name=db_name)

        if not is_sql:
            cmd += "| gzip -9 -c "
            file_name += ".gz"

        cmd += " > {file_name}"

        msg = f"Dumping the '{db_name}' database 📥"
        click.echo(msg)
        os.system(cmd.format(db_name=db_name, file_name=file_name))

    def db_import(self, sql_path, db_name):
        if not self.db_exists(db_name):
            raise DbExistsError(self.MSG_DB_EXISTS.format(db_name=db_name))

        if not os.path.isfile(sql_path):
            raise SqlFileNotExistsError(f"SQL file '{sql_path}' was not found")

        mime = magic.from_file(sql_path, mime=True)

        cmd = ''

        if mime == 'application/gzip':
            cmd += 'z'

        cmd += 'cat "{sql_path}" | mysql {db_dest}'
        os.system(cmd.format(sql_path=sql_path, db_dest=db_name))

    def db_remove_definer(self, sql_path):
        if not os.path.isfile(sql_path):
            raise SqlFileNotExistsError(f"SQL file '{sql_path}' was not found")

        tmp_path = "/tmp/{hash}".format(hash=uuid.uuid4().hex)

        mime = magic.from_file(sql_path, mime=True)

        if mime == 'application/gzip':
            cmd = list()
            cmd.append(rf'zcat "{sql_path}" | sed "s/DEFINER=[^*]*\*/\*/g" > "{tmp_path}"')
            cmd.append(rf'gzip -9 -c "{tmp_path}" > "{sql_path}"')
            cmd.append(rf'rm "{tmp_path}"')

            os.system(' && '.join(cmd))
            return

        os.system(rf'sed -i "s/DEFINER=[^*]*\*/\*/g" "{sql_path}"')

    def sql_file_complete(self, ctx, args, incomplete):
        files = glob('*.sql') + glob('*.sql.gz')
        files.append(args)
        return [k for k in files if incomplete in k]

    def databases_complete(self, ctx, args, incomplete):
        databases = self.db_list()
        return [k for k in databases if incomplete in k]



import click
import os
import wt_plus.core


services = click.Group(name='services')


@services.command()
def start():
    wt_plus.core.site.http.start()
    wt_plus.core.site.php.start_all()
    #wt_plus.core.db.start()
    wt_plus.core.mailhog.start()


@services.command()
def stop():
    wt_plus.core.site.http.stop()
    wt_plus.core.site.php.stop_all()
    #wt_plus.core.db.stop()
    wt_plus.core.mailhog.stop()


@services.command()
def restart():
    wt_plus.core.site.http.restart()
    wt_plus.core.site.php.restart_all()
    #wt_plus.core.db.restart()
    wt_plus.core.mailhog.restart()
    wt_plus.core.redis.restart()


@services.command()
def enable():
    wt_plus.core.site.http.enable()
    wt_plus.core.site.php.enable_all()
    #wt_plus.core.db.enable()
    wt_plus.core.mailhog.enable()
    wt_plus.core.redis.enable()


@services.command()
def disable():
    wt_plus.core.site.http.disable()
    wt_plus.core.site.php.disable_all()
    #wt_plus.core.db.disable()
    wt_plus.core.mailhog.disable()
    wt_plus.core.redis.disable()


@services.command()
def configure():
    os.system(r'sudo apt install -y oathtool')
    os.system(fr'cp {wt_plus.core.config.wt_path}/config/recaptcha_keys.json {wt_plus.core.config.wt_path}/config/recaptcha_keys.json')
    wt_plus.core.site.configure()
    wt_plus.core.site.host.configure()
    wt_plus.core.site.http.configure()
    wt_plus.core.site.php.configure()
    wt_plus.core.db.configure()
    wt_plus.core.mailhog.configure()
    wt_plus.core.redis.configure()
    wt_plus.core.rabbitmq.configure()
    wt_plus.core.site.dump_config()



import os
from jinja2 import Template
import subprocess


class Php(object):
    def __init__(self, config):
        self.config = config
        self.site = None

    def configure(self):
        if not os.path.exists('/etc/apt/sources.list.d/ondrej-ubuntu-php-focal.list'):
            subprocess.run(['sudo apt install -y software-properties-common'], shell=True, executable='/bin/bash')
            subprocess.run(['sudo add-apt-repository -y ppa:ondrej/php'], shell=True, executable='/bin/bash')
            subprocess.run(['sudo apt update -y'], shell=True, executable='/bin/bash')

        cmd = 'sudo apt install -y php{5.6,7.1,7.2,7.3,7.4,8.0}-{common,gmp,curl,soap,bcmath,intl,mbstring,xmlrpc,mysql,gd,xml,cli,zip,dev,ldap,fpm,imap,apcu,imagick,mcrypt,yaml,oauth,xdebug}'
        subprocess.run([cmd], shell=True, executable='/bin/bash')

        subprocess.run(['sudo apt install -y composer'], shell=True, executable='/bin/bash')

        for version in self.versions():
            pool_path = f'{self.config.wt_config_path}/fpm/{version}/pool.d'

            if not os.path.exists(pool_path):
                cmd = f'mkdir -p {pool_path}'
                os.system(cmd)

            create = f'[ $(grep -c "include={pool_path}/\*.conf" /etc/php/{version}/fpm/php-fpm.conf) == "0" ] && echo "\ninclude={pool_path}/*.conf" >> /etc/php/{version}/fpm/php-fpm.conf'
            os.system(f'sudo /bin/bash -c \'{create}\'')
            os.system(f'echo "sendmail_path=/usr/local/bin/mhsendmail\n" | sudo tee /etc/php/{version}/mods-available/mailhog.ini')

        os.system('sudo phpenmod mailhog')
        os.system('sudo phpdismod -s cli xdebug')

    def start_all(self):
        for version in self.versions():
            self.service('start', version)

    def stop_all(self):
        for version in self.versions():
            self.service('stop', version)

    def restart_all(self):
        for version in self.versions():
            self.restart(version)

    def disable_all(self):
        for version in self.versions():
            self.disable(version)

    def enable_all(self):
        for version in self.versions():
            self.enable(version)

    def restart(self, version):
        self.service('restart', version)

    def enable(self, version):
        self.service('enable', version)

    def disable(self, version):
        self.service('disable', version)

    def service(self, option, php_version):
        os.system(f'sudo systemctl {option} php{php_version}-fpm.service')

    def versions(self):
        return ['5.6', '7.1', '7.2', '7.3', '7.4', '8.0']

    def cli_version(self, version):
        cmd = [
            "sudo update-alternatives --set php /usr/bin/php{version}",
            "sudo update-alternatives --set phar /usr/bin/phar{version}",
            "sudo update-alternatives --set phar.phar /usr/bin/phar.phar{version}",
            "sudo update-alternatives --set phpize /usr/bin/phpize{version}",
            "sudo update-alternatives --set php-config /usr/bin/php-config{version}"
        ]

        os.system(' && '.join(cmd).format(version=version))

    def current_version(self, site_id):
        return self.config.get('sites').get(site_id).get('php').get('version')

    def xdebug(self, status, remote_autostart, sapi_name, php_version, current_site_id):
        if php_version is None:
            php_version = self.current_version(current_site_id)

        self.phpmod('xdebug', status, sapi_name, php_version)

        self.restart(php_version)

    def phpmod(self, mod, status, sapi_name, php_version):
        if php_version is None:
            php_version = self.current_version(self.config.current_site_id)

        if status == 'on':
            os.system(f'sudo phpenmod -v {php_version} -s {sapi_name} {mod}')
        elif status == 'off':
            os.system(f'sudo phpdismod -v {php_version} -s {sapi_name} {mod}')

        self.restart(php_version)

    def parser_config(self, site_id):
        pool_template = Template(open(f'{self.config.wt_path}/templates/php/pool.conf.jinja').read())

        site = self.config.get('sites').get(site_id)

        if site is None:
            return

        php_config = site.get('php')
        user = self.config.user

        if not php_config:
            return

        php_version = php_config.get('version')

        php_flag = php_config.get('php_flag')
        php_flag = dict(php_flag) if php_flag else dict()
        if 'display_errors' not in php_flag:
            php_flag['display_errors'] = 'on'

        php_admin_value = php_config.get('php_admin_value')
        php_admin_value = dict(php_admin_value) if php_admin_value else dict()
        php_admin_value['error_log'] = f'{self.config.wt_config_path}/log/{site_id}-fpm-php.log'
        if 'xdebug.mode' not in php_admin_value:
            php_admin_value['xdebug.mode'] = 'debug'
        if 'xdebug.remote_enable' not in php_admin_value:
            php_admin_value['xdebug.remote_enable'] = True

        pool_template.stream(
            site_id=site_id,
            user=user,
            php_version=php_version,
            php_flag=php_flag,
            php_admin_value=php_admin_value).dump(f'{self.config.wt_config_path}/fpm/{php_version}/pool.d/{site_id}.conf')

    def dump_config(self):
        os.system(f'find {self.config.wt_config_path}/fpm/ -name "*.conf" -delete')

        for site_id in self.config.get('sites'):
            self.parser_config(site_id)

    def php_versions(self, ctx, args, incomplete):
        versions = ['5.6', '7.1', '7.2', '7.3', '7.4', '8.0']
        return [k for k in versions if incomplete in k]



import click
import subprocess
import wt_plus.core


def _app_import():
    subprocess.check_output(['n98 app:config:import'], shell=True)


def _cache_flush():
    subprocess.check_output(['n98 cache:flush'], shell=True)


def mage_versions(ctx, args, incomplete):
    return [k for k in wt_plus.core.site.magento2.versions if incomplete in k]


magento2 = click.Group(name='magento2')


@magento2.command('install')
@click.argument('version', default='2.4.3', type=click.STRING, autocompletion=mage_versions)
@click.option('--firstname', default='Wt')
@click.option('--admin_user', default='admin')
@click.option('--admin_password', default='admin123')
@click.option('--lastname', default='Env')
@click.option('--email', default=f'admin@{wt_plus.core.site.default_site_id()}.test')
@click.option('--base-url', default=f'http://{wt_plus.core.site.default_site_id()}.test')
@click.option('--with-sample-data', is_flag=True, default=False)
def install(version, admin_user, admin_password, firstname, lastname, email, base_url, with_sample_data):
    try:
        site_id = wt_plus.core.site.current_site_id
        wt_plus.core.site.magento2.install(site_id, version, admin_user, admin_password, firstname, lastname, email,
                                          base_url, with_sample_data)
        click.echo(base_url)
    except wt_plus.core.SiteNotExistsError as e:
        click.echo(e)


@magento2.command()
@click.argument('src')
@click.argument('dist', default=f'{wt_plus.core.site.default_site_id()}.test')
@click.argument('db_name', default=wt_plus.core.site.default_site_id())
def change_base_url(src, dist, db_name):
    try:
        wt_plus.core.site.magento2.change_base_url(src, dist, db_name)
        click.echo(f"Changed domain 🔥")
    except wt_plus.core.SiteNotExistsError as e:
        click.echo(e)


@magento2.command()
def smtp_disable():
    try:
        subprocess.check_output(['n98 config:set transactional_emails/ddg_transactional/enabled 0'], shell=True)
    except:
        pass


@magento2.command('dev-config')
def dev_config():
    wt_plus.core.site.magento2.dev_config()


@magento2.command()
def version():
    click.echo(wt_plus.core.site.magento2.version)


@magento2.command()
@click.argument('admin_user', default="admin")
def token(admin_user):
    wt_plus.core.site.magento2.token(admin_user)


@magento2.command('sample-data')
def sample_data():
    wt_plus.core.site.magento2.install_sample_data()


@magento2.command()
def clean_static():
    wt_plus.core.site.magento2.clean_static()
    click.echo('Cleaning completed 🧹️')


@magento2.command('tfa-config')
@click.argument('admin_user', default="admin")
@click.argument('secret', default="MFRGGZDF")
def twofactorauth_change(admin_user, secret):
    wt_plus.core.site.magento2.twofactorauth(admin_user, secret)


@magento2.command()
@click.argument('status', type=click.Choice(['on', 'off'], case_sensitive=False))
def recaptcha(status):
    status = int(status == 'on')
    wt_plus.core.site.magento2.recaptcha(status)



import os
import re
from jinja2 import Template


class Http(object):

    def __init__(self, config):
        self.config = config

    def install(self):
        os.system(rf'mkdir -p {self.config.wt_config_path}/sites')
        os.system(rf'mkdir -p {self.config.wt_config_path}/log')
        os.system(rf'mkdir -p {self.config.wt_config_path}/config')
        os.system(r'sudo apt install -y apache2')
        os.system(r'sudo a2enmod ssl')
        os.system(r'sudo a2enmod actions fcgid alias proxy_fcgi')
        os.system(r'sudo a2enmod rewrite')
        os.system(r'sudo a2enmod headers')
        os.system(r'sudo a2enmod expires')
        os.system(r'sudo a2enmod include')
        os.system(r'sudo a2enmod proxy')
        os.system(r'sudo a2enmod proxy_http')
        os.system(r'sudo a2enmod proxy_wstunnel')

    def configure(self):
        self.install()

        # Add custom wt config home user
        config_wt = f'Timeout -1\n\nIncludeOptional {self.config.wt_config_path}/sites/*.conf\n'
        os.system(fr'sudo /bin/bash -c "echo \"{config_wt}\" > /etc/apache2/conf-available/wt.conf"')

        # Configuring envvars
        os.system(fr'sudo sed -i "s,^\(export APACHE_RUN_USER=\).*,\1{self.config.user},g" /etc/apache2/envvars')
        os.system(fr'sudo sed -i "s,^\(export APACHE_RUN_GROUP=\).*,\1{self.config.user},g" /etc/apache2/envvars')

        os.system(r"sudo sed -i 's/Listen 80/Listen 0.0.0.0:80/g' /etc/apache2/ports.conf")
        os.system(r"sudo sed -i 's/Listen 443/Listen 0.0.0.0:443/g' /etc/apache2/ports.conf")

        os.system('sudo a2enconf wt.conf')

        self.restart()

    def start(self):
        os.system('sudo systemctl start apache2')

    def stop(self):
        os.system('sudo systemctl stop apache2')

    def restart(self):
        os.system('sudo systemctl stop apache2 && sudo systemctl start apache2')

    def enable(self):
        os.system('sudo systemctl enable apache2')

    def disable(self):
        os.system('sudo systemctl disable apache2')

    def filter_proxies(self, proxies):
        proxies_pass = list()
        proxies_match = list()

        if proxies is not None:
            match_test = re.compile(r'.*[\[(^$].*')
            for _ in proxies:
                if match_test.match(_.split(' ')[0]):
                    proxies_match.append(_)
                else:
                    proxies_pass.append(_)

        return [proxies_pass, proxies_match]

    def dump_config(self):
        virtualhost_tpl = Template(open(self.config.wt_path + '/templates/apache/virtualhost.conf.jinja').read(),
                                   trim_blocks=True, lstrip_blocks=True)
        virtualhost_302_tpl = Template(
            open(self.config.wt_path + '/templates/apache/virtualhost-302.conf.jinja').read(), trim_blocks=True,
            lstrip_blocks=True)

        sites = self.config.get_config('sites')
        os.system(f'rm -rf {self.config.wt_config_path}/sites/*')

        for site in sites:
            site_config = sites.get(site)

            if site_config is None:
                continue

            path = site_config.get('path')
            public_path = site_config.get('public_path')
            custom = site_config.get('custom')
            aliases = site_config.get('alias')
            proxies = site_config.get('proxy')
            secure = site_config.get('secure')
            crt_path = f'{self.config.certificate_path}/{site}.crt'
            key_path = f'{self.config.certificate_path}/{site}.key'
            php_version = self.config.get_site_config('php/version', site)
            log_path = f'{self.config.wt_config_path}/log'

            if path is not None and not os.path.exists(path):
                continue

            server_name = f"{site}.test"
            if aliases is not None:
                aliases = ' '.join(aliases)

            proxies_pass, proxies_match = self.filter_proxies(proxies)

            public_html = str(path).rstrip('/')

            if public_path is not None:
                public_html += '/' + public_path

            virtualhost_tpl.stream(path='/var/www/html').dump(f'{self.config.wt_config_path}/sites/000-default.conf')
            virtualhost_tpl.stream(
                secure=True,
                path='/var/www/html',
                crt_path='/etc/ssl/certs/ssl-cert-snakeoil.pem',
                key_path='/etc/ssl/private/ssl-cert-snakeoil.key'
            ).dump(f'{self.config.wt_config_path}/sites/000-default-ssl.conf')

            if secure and os.path.exists(crt_path) and os.path.exists(key_path):
                virtualhost_302_tpl.stream(
                    server_name=server_name,
                    aliases=aliases).dump(f'{self.config.wt_config_path}/sites/{site}.conf')

                virtualhost_tpl.stream(
                    site_id=site,
                    log_path=log_path,
                    path=public_html,
                    server_name=server_name,
                    proxies_pass=proxies_pass,
                    proxies_match=proxies_match,
                    secure=secure,
                    crt_path=crt_path,
                    key_path=key_path,
                    custom=custom,
                    aliases=aliases,
                    php_version=php_version).dump(f'{self.config.wt_config_path}/sites/{site}-ssl.conf')
            else:
                virtualhost_tpl.stream(
                    site_id=site,
                    path=public_html,
                    log_path=log_path,
                    server_name=server_name,
                    proxies_pass=proxies_pass,
                    proxies_match=proxies_match,
                    custom=custom,
                    aliases=aliases,
                    php_version=php_version).dump(f'{self.config.wt_config_path}/sites/{site}.conf')



import os


class MailHog(object):
    def __init__(self, config, site):
        self.config = config
        self.site = site

    def configure(self):
        if not os.path.exists('/usr/local/bin'):
            os.system(f'sudo mkdir -p /usr/local/bin')

        if not os.path.exists('/usr/local/bin/mailhog'):
            os.system('sudo wget https://github.com/mailhog/MailHog/releases/download/v1.0.1/MailHog_linux_amd64'
                      ' -O /usr/local/bin/mailhog')
            os.system('sudo chmod +x /usr/local/bin/mailhog')

        if not os.path.exists('/usr/local/bin/mhsendmail'):
            os.system('sudo wget https://github.com/mailhog/mhsendmail/releases/download/v0.2.0/mhsendmail_linux_amd64'
                      ' -O /usr/local/bin/mhsendmail')
            os.system('sudo chmod +x /usr/local/bin/mhsendmail')

        if not self.site.is_site_exists('proxy'):
            self.config.set_site_config('proxy', [
                '/api/v2/websocket ws://localhost:8025/api/v2/websocket',
                '/ http://localhost:8025/'
            ], 'mailhog')
            self.site.secure('mailhog')

        mailhog_service = """
            sudo tee /etc/systemd/system/mailhog.service <<EOL
[Unit]
Description=MailHog
After=network.target
[Service]
ExecStart=/usr/bin/env /usr/local/bin/mailhog > /dev/null 2>&1 &
[Install]
WantedBy=multi-user.target
EOL
"""
        if not os.path.exists('/etc/systemd/system/mailhog.service'):
            os.system(mailhog_service)
            os.system('sudo systemctl daemon-reload')

        os.system('sudo systemctl restart mailhog.service')

    def start(self):
        os.system('sudo systemctl start mailhog.service')

    def stop(self):
        os.system('sudo systemctl stop mailhog.service')

    def restart(self):
        os.system('sudo systemctl restart mailhog.service')

    def enable(self,):
        os.system('sudo systemctl enable mailhog.service')

    def disable(self):
        os.system('sudo systemctl disable mailhog.service')



import os
import subprocess
import click
import wt_plus.core

from prettytable import PrettyTable


def php_versions(ctx, args, incomplete):
    versions = ['5.6', '7.1', '7.2', '7.3', '7.4', '8.0']
    return [k for k in versions if incomplete in k]


def status_complete(ctx, args, incomplete):
    status = ['on', 'off']
    return [k for k in status if incomplete in k]


def sapi_name_complete(ctx, args, incomplete):
    status = ['cli', 'fpm', 'all']
    return [k for k in status if incomplete in k]


def framework_complete(ctx, args, incomplete):
    versions = ['magento2']
    return [k for k in versions if incomplete in k]


wt = click.Group(name='wt')


@wt.command('about')
def about():
    click.echo("This is a CLI built with Click 🔥")


@wt.command('xdebug')
@click.argument('status', type=click.STRING, autocompletion=status_complete)
@click.option('--remote_autostart', default=False, is_flag=True)
@click.option('-s', '--sapi-name', type=str, default='fpm', autocompletion=sapi_name_complete, help="Sapi name")
@click.option('-v', '--php-version', type=str, autocompletion=php_versions, help="PHP version")
def xdebug(status, remote_autostart, sapi_name, php_version):
    current_site_id = wt_plus.core.site.current_site_id
    wt_plus.core.site.php.xdebug(status, remote_autostart, sapi_name, php_version, current_site_id)


@wt.command()
@click.argument('path', type=click.STRING, nargs=-1)
@click.option('--idekey', type=str, default='PHPSTORM', help="idekey")
def phpx(path, idekey):
    path = ' '.join(path)
    os.system(f'XDEBUG_CONFIG="idekey={idekey}" php -dzend_extension=xdebug.so -dxdebug.mode=debug {path}')


@wt.command()
@click.argument('path', type=click.STRING, autocompletion=wt_plus.core.config.config_path_complete)
@click.argument('value', type=click.STRING, autocompletion=wt_plus.core.config.config_value_complete, required=False)
@click.argument('site_id', default=wt_plus.core.site.default_site_id())
def config(path, value, site_id):
    if path not in wt_plus.core.config.public_configs:
        click.echo('Não disponível')
        return

    if value is None:
        click.echo(wt_plus.core.config.get_site_config(path, site_id))
        return

    if path == 'php/php_admin_value/auto_prepend_file':
        value = f'{wt_plus.core.config.current_path}/{value}'

    wt_plus.core.config.set_site_config(path, value, site_id)
    wt_plus.core.site.dump_config()


@wt.command()
@click.argument('path', type=click.STRING, autocompletion=wt_plus.core.config.config_path_complete)
@click.argument('site_id', default=wt_plus.core.site.default_site_id())
def unconfig(path, site_id):
    if path not in wt_plus.core.config.public_configs:
        click.echo('Não disponível')
        return

    click.echo(wt_plus.core.config.unset_site_config(path, site_id))
    wt_plus.core.config.save()


@wt.command('self-config')
def self_config():
    os.system(f"{os.getenv('EDITOR') or 'nano'} {wt_plus.core.config.file_config}")
    wt_plus.core.config.reload()
    wt_plus.core.site.dump_config()


@wt.command()
@click.argument('site_id', default=wt_plus.core.site.default_site_id())
def log(site_id):
    if not wt_plus.core.site.is_site_exists(site_id):
        click.echo('Site não existe')

    os.system(f'tail -f {wt_plus.core.config.wt_config_path}/log/{site_id}-*')


@wt.command()
@click.argument('version', required=False, type=click.FLOAT, autocompletion=php_versions)
@click.argument('site_id', default=wt_plus.core.site.default_site_id())
@click.option('--update-cli', default=False, is_flag=True, help='Update cli php')
def use(version, site_id, update_cli):
    try:
        if version is None:
            version = wt_plus.core.config.get_site_config('php/version', site_id) or 7.3
            update_cli = True

        wt_plus.core.site.use(version, site_id, update_cli)
        wt_plus.core.site.dump_config()
        click.echo(f"PHP {version} {site_id} 🔥")
    except wt_plus.core.SiteNotExistsError as e:
        click.echo(e)


@wt.command()
def links():
    table = PrettyTable(['Site', 'SSL', 'PHP', 'URL', 'Path/[public_html]'])
    table.align = 'l'

    sites = wt_plus.core.config.get('sites')

    for site in sites:
        curren_site = sites.get(site)

        ssl, http = ['', 'http']
        if wt_plus.core.config.get_site_config('secure', site) is True:
            ssl, http = ['X', 'https']

        url = f'{http}://{site}.test'

        if curren_site.get('alias'):
            for site_alias in curren_site.get('alias'):
                url += f'\n{http}://{site_alias}'

        php_version = wt_plus.core.config.get_site_config('php/version', site) or ''
        public_path = wt_plus.core.config.get_site_config('public_path', site) or ''
        path = wt_plus.core.config.get_site_config('path', site) or ''

        if not path == '' and not public_path == '':
            path += f'[{public_path}]'

        table.add_row([site, ssl, php_version, url, path])

    click.echo(table)


@wt.command()
@click.argument('site', default=wt_plus.core.site.default_site_id())
@click.option('--framework', default=None, autocompletion=framework_complete)
@click.option('--unsecure', default=False, is_flag=True)
def link(site, framework, unsecure):
    try:
        wt_plus.core.site.add(site, framework, unsecure)
        wt_plus.core.site.dump_config()
        click.echo(f"'{site}' run 🔥")
    except wt_plus.core.SiteExistsError as e:
        click.echo(e)


@wt.command()
@click.argument('site_id', default=wt_plus.core.site.default_site_id(), autocompletion=wt_plus.core.site.sites_complete)
def unlink(site_id):
    try:
        wt_plus.core.site.remove(site_id)
        wt_plus.core.site.dump_config()
        click.echo(f"'{site_id}' remove 💦")
    except wt_plus.core.SiteNotExistsError as e:
        click.echo(e)


@wt.command()
@click.argument('site_id', default=wt_plus.core.site.default_site_id(), autocompletion=wt_plus.core.site.sites_complete)
def secure(site_id):
    wt_plus.core.site.secure(site_id)
    wt_plus.core.site.dump_config()


@wt.command()
@click.argument('site_id', default=wt_plus.core.site.default_site_id(), autocompletion=wt_plus.core.site.sites_complete)
def unsecure(site_id):
    wt_plus.core.site.unsecure(site_id)
    wt_plus.core.site.dump_config()


@wt.command()
@click.argument('name')
def alias(name):
    try:
        wt_plus.core.site.add_alias(wt_plus.core.site.current_site_id, name)
        click.echo(f"'{name}' adicionado 🔥")
    except wt_plus.core.SiteNotExistsError as e:
        click.echo(e)


@wt.command()
@click.argument('origin')
@click.argument('target')
@click.argument('site_id', default=wt_plus.core.site.default_site_id(), autocompletion=wt_plus.core.site.sites_complete)
def proxy(origin, target, site_id):
    try:
        wt_plus.core.site.add_proxy(site_id, origin, target)
        wt_plus.core.site.dump_config()
        click.echo(f"'{origin}' -> '{target}' adicionado 🔥")
    except wt_plus.core.SiteNotExistsError as e:
        click.echo(e)
    except wt_plus.core.ProxyExistsError as e:
        click.echo(e)


@wt.command()
@click.argument('origin')
@click.argument('site_id', default=wt_plus.core.site.default_site_id(), autocompletion=wt_plus.core.site.sites_complete)
def unproxy(origin, site_id):
    try:
        wt_plus.core.site.rm_proxy(site_id, origin)
        wt_plus.core.site.dump_config()
        click.echo(f"'{origin}' removido 💦")
    except wt_plus.core.SiteNotExistsError as e:
        click.echo(e)


@wt.command()
@click.argument('name', autocompletion=wt_plus.core.site.aliases_complete)
@click.argument('site_id', default=wt_plus.core.site.default_site_id(), autocompletion=wt_plus.core.site.sites_complete)
def unalias(name, site_id):
    try:
        wt_plus.core.site.del_alias(site_id, name)
        wt_plus.core.site.dump_config()
        click.echo(f"'{name}' removido 💦")
    except wt_plus.core.SiteNotExistsError as e:
        click.echo(e)


@wt.command()
def reload():
    wt_plus.core.config.reload()
    wt_plus.core.site.http.dump_config()


@wt.command('open')
@click.argument('path', default='.')
@click.option('-d', default=False, is_flag=True, help='Chooose')
def open_(path, d):
    click.launch("{path}".format(path=path), locate=True)


@wt.command()
@click.argument('path', type=click.Path(exists=True), default=os.getcwd())
@click.argument('path_compare', type=click.Path(exists=True), required=False, default=None)
def phpstorm(path, path_compare):
    if path_compare:
        subprocess.run([f'nohup phpstorm diff {path} {path_compare} > /dev/null 2>&1 & disown'],
                       shell=True, executable='/bin/bash')
    else:
        subprocess.run([f'nohup phpstorm {path} > /dev/null 2>&1 & disown'],
                       shell=True, executable='/bin/bash')


@wt.command(hidden=True)
@click.argument('shell', type=click.Choice(['bash', 'zsh', 'fish'], case_sensitive=False))
def completions(shell):
    os.system(f'LC_ALL=C _WT_COMPLETE={shell}_source wt')



import os
import json
from jinja2 import Template


class SiteExistsError(TypeError):
    pass


class SiteNotExistsError(TypeError):
    pass


class ProxyExistsError(TypeError):
    pass


class ProxyNotExistsError(TypeError):
    pass


class Site:
    MSG_SITE_EXISTS = "{site_id} site already exists! 😖"
    MSG_SITE_NOT_EXISTS = "Site '{site_id}' não existe! 😖"
    MSG_PROXY_EXISTS = "{proxy} proxy already exists! 😖"

    # TODO: fix orrible design
    def __init__(self, config, host, http, php, db, magento2):
        self.config = config
        self.db = db
        self.http = http
        self.php = php

        self.magento2 = magento2
        self.magento2.site = self
        self.magento2.php = php
        self.magento2.config = config
        self.magento2.db = db

        self.host = host

        self.ca_path = f'{self.config.wt_config_path}/ca-certificates'
        self.ca_crt_path = f'{self.ca_path}/WtEnvCASelfSigned.crt'
        self.ca_key_path = f'{self.ca_path}/WtEnvCASelfSigned.key'
        self.ca_srl_path = f'{self.ca_path}/WtEnvCASelfSigned.srl'
        self.cert_path = f'{self.config.wt_config_path}/certificates'

    def install(self):
        os.system(r'sudo apt install libnss3-tools')

    def configure(self):
        self.install()

    def default_site_id(self, ignore_alias=False, onlyexists=False):
        """
        TODO: Set to consider only the full site path
        """
        project_id = os.path.basename(os.path.splitext(os.getcwd())[0][1:])
        alias = self.config.get_config(f'wt/alias/{project_id}')
        if alias and not ignore_alias:
            project_id = alias
        site = self.config.get_config(f'sites/{project_id}')

        if not site and onlyexists is True:
            return

        return project_id

    @property
    def current_site_id(self):
        if not self.is_site_exists(self.default_site_id(onlyexists=True)):
            raise SiteNotExistsError(self.MSG_SITE_NOT_EXISTS.format(site_id=self.default_site_id()))

        return self.default_site_id(onlyexists=True)

    def find_site_by_id(self, site_id):
        if not self.is_site_exists(site_id):
            raise SiteNotExistsError(self.MSG_SITE_NOT_EXISTS.format(site_id=site_id))

        return self.config.get('sites').get(site_id)

    def site_path(self):
        vhost = self.config.get('sites').get(self.default_site_id())

        if vhost is False:
            raise SiteNotExistsError(self.MSG_SITE_NOT_EXISTS.format(site_id=self.default_site_id()))

        return vhost.get('path').rstrip('/')

    def is_site_exists(self, site_id) -> bool:
        if self.config.get('sites').get(site_id):
            return True

        return False

    def sites(self):
        sites_config = self.config.get('sites')
        sites = list()

        for site in sites_config:
            sites.append(site)

        return sites

    def use(self, version, site_id, update_cli=None):
        if update_cli:
            self.php.cli_version(version)

        self.config.set_site_config('php/version', version, site_id)

    def add(self, site_id, framework, unsecure):
        if self.is_site_exists(site_id):
            raise SiteExistsError(self.MSG_SITE_EXISTS.format(site_id=site_id))

        php_version = self.config.get_config('php/version') or 7.3
        public_path = ''

        self.config.set_site_config('path',  os.getcwd() + "/", site_id)
        self.config.set_site_config('php', {"version": php_version}, site_id)

        if framework is not None:
            self.config.set_site_config('framework', framework, site_id)

            if framework == 'magento2':
                public_path = 'pub'

        self.config.set_site_config('public_path', public_path, site_id)

        if not unsecure:
            self.config.set_site_config('secure', True, site_id)

        real_path = self.default_site_id(ignore_alias=True)

        if site_id != real_path:
            self.config.set_config(f'wt/alias/{real_path}', site_id)

        if not unsecure:
            self.secure(site_id)

    def remove(self, site_id):
        if not self.is_site_exists(site_id):
            raise SiteNotExistsError(self.MSG_SITE_NOT_EXISTS.format(site_id=site_id))

        secure = self.config.get_site_config('secure', site_id)
        if not secure:
            self.unsecure(site_id)

        self.config.get('sites').pop(site_id, None)
        self.config.get('wt').get('alias').pop(self.default_site_id(ignore_alias=True), None)

    def add_alias(self, site_id, name):
        if not self.is_site_exists(site_id):
            raise SiteNotExistsError(self.MSG_SITE_NOT_EXISTS.format(site_id=site_id))

        for site in self.sites():
            aliases = self.config.get('sites').get(site).get('alias')

            if aliases is None:
                continue

            for alias in aliases:
                if alias == name:
                    aliases.remove(name)

        alias = self.config.get('sites').get(site_id).get('alias')
        if alias is None:
            alias = self.config.get('sites').get(site_id)['alias'] = list()

        alias.append(name)

    def del_alias(self, site_id, name):
        if not self.is_site_exists(site_id):
            raise SiteNotExistsError(self.MSG_SITE_NOT_EXISTS.format(site_id=site_id))

        aliases = self.config.get('sites').get(site_id).get('alias')

        if aliases is not None:
            for alias in aliases:
                if alias == name:
                    aliases.remove(name)

    def add_proxy(self, site_id, origin, target):
        if self.is_site_exists(site_id):
            site = self.find_site_by_id(site_id)
        else:
            site = self.config.get('sites')[site_id] = dict()

        proxies = site.get('proxy')

        if proxies is None:
            proxies = site['proxy'] = list()

        for proxy in proxies:
            if proxy.split(' ')[0] == origin:
                raise ProxyExistsError(self.MSG_PROXY_EXISTS.format(proxy=proxy))

        proxies.append(f'{origin} {target}')

    def rm_proxy(self, site_id, origin):
        site = self.find_site_by_id(site_id)

        proxies = site.get('proxy')

        if proxies is None:
            return

        _ = list()
        for proxy in proxies:
            if proxy.split(' ')[0] != origin:
                _.append(proxy)

        if _:
            site['proxy'] = _
        else:
            site.pop('proxy', None)

    def _create_ca(self):
        if os.path.exists(self.ca_crt_path) and os.path.exists(self.ca_key_path):
            return

        if not os.path.exists(self.ca_path):
            os.system(f'mkdir -p {self.ca_path}')

        o_name = 'Wt Env CA Self Signed Organization'
        c_name = 'Wt Env CA Self Signed CN'
        email = 'rootcertificate@wt_plus.test'

        cmd = f'openssl req -new -newkey rsa:2048 -days 730 -nodes -x509' \
              f' -subj "/C=/ST=/O={o_name}/localityName=/commonName={c_name}/organizationalUnitName=Developers/emailAddress={email}/"' \
              f' -keyout "{self.ca_key_path}" -out "{self.ca_crt_path}"'
        os.system(cmd)

        self._trust_ca()

    def _trust_ca(self):
        cmd = f'certutil -d sql:$HOME/.pki/nssdb -D -n "WtEnvCAcert.org"'
        os.system(cmd)

        cmd = f'certutil -d sql:$HOME/.pki/nssdb -A -t TC -n "WtEnvCAcert.org" -i {self.ca_crt_path}'
        os.system(cmd)

        cmd = f'sudo ln -sf {self.ca_crt_path} /usr/local/share/ca-certificates/WtEnvCASelfSigned.crt'
        os.system(cmd)

        mozilla_policy = {
            "policies": {
                "Certificates": {
                    "Install": [
                        "/usr/local/share/ca-certificates/WtEnvCASelfSigned.crt"
                    ]
                }
            }
        }

        os.system('sudo mkdir -p /usr/lib/firefox/distribution')
        os.system('sudo rm /usr/lib/firefox/distribution/policies.json')
        os.system(f"sudo bash -c 'echo {json.dumps(json.dumps(mozilla_policy))} > /usr/lib/firefox/distribution/policies.json'")

        cmd = 'sudo update-ca-certificates --fresh'
        os.system(cmd)

    def secure(self, site_id):
        self._create_ca()

        if not self.is_site_exists(site_id):
            raise SiteNotExistsError(self.MSG_SITE_NOT_EXISTS.format(site_id=site_id))

        aliases = self.config.get('sites').get(site_id).get('alias')

        domains = list()
        domains.append(f'{site_id}.test')
        domains.append(f'*.{site_id}.test')

        if aliases is not None:
            for alias in aliases:
                domain = ".".join(alias.split('.')[-2:])
                if domain not in domains:
                    domains.append(domain)
                    domains.append(f'*.{domain}')

        url = f'{site_id}.test'

        if not os.path.exists(self.cert_path):
            os.system(f'mkdir -p {self.cert_path}')

        key_path = f'{self.cert_path}/{site_id}.key'
        csr_path = f'{self.cert_path}/{site_id}.csr'
        crt_path = f'{self.cert_path}/{site_id}.crt'
        conf_path = f'{self.cert_path}/{site_id}.conf'

        openssl_template = Template(open(self.config.wt_path + '/templates/openssl.conf.jinja').read(),
                                    trim_blocks=True, lstrip_blocks=True)
        openssl_template.stream(site=site_id, domains=enumerate(domains)).dump(conf_path)

        # Generate private key
        cmd = f'openssl genrsa -out "{key_path}" 2048'
        os.system(cmd)

        # Create signin request
        cmd = f'openssl req -new -key {key_path} -out {csr_path} -subj "/C=/ST=/O=/localityName=/commonName=*.{url}/organizationalUnitName=/emailAddress=/" -config {conf_path} -passin pass:'
        os.system(cmd)

        ca_srl_param = f'-CAserial "{self.ca_srl_path}"'

        if not os.path.exists(self.ca_srl_path):
            ca_srl_param += ' -CAcreateserial'

        cmd = f'openssl x509 -req -sha256 -days 730 -CA "{self.ca_crt_path}" -CAkey "{self.ca_key_path}" {ca_srl_param} -in "{csr_path}" -out "{crt_path}" -extensions v3_req -extfile "{conf_path}"'
        os.system(cmd)

        self.config.get('sites').get(site_id)['secure'] = True

        framework = self.config.get('sites').get(site_id).get('framework')

        if framework == 'magento2' and self.magento2.version is not None:
            self.magento2.dev_config()

    def unsecure(self, site_id):
        if not self.is_site_exists(site_id):
            raise SiteNotExistsError(self.MSG_SITE_NOT_EXISTS.format(site_id=site_id))

        self.config.get('sites').get(site_id).pop('secure', None)
        os.system(f'find {self.cert_path} -name "{site_id}.test*" -type f -delete')

        framework = self.config.get('sites').get(site_id).get('framework')

        if framework == 'magento2':
            self.magento2.dev_config()

    def dump_config(self):
        self.config.save()
        self.php.dump_config()
        self.http.dump_config()
        self.host.dump_config()
        self.php.restart_all()
        self.http.restart()

    def aliases_complete(self, ctx, args, incomplete):
        aliases = self.config.get('sites').get(self.current_site_id).get('alias')
        if aliases is None:
            aliases = list()

        return [k for k in aliases if incomplete in k]

    def sites_complete(self, ctx, args, incomplete):
        sites = self.sites()
        return [k for k in sites if incomplete in k]



import os


class Host(object):
    def __init__(self, config):
        self.config = config

    def configure(self):
        create = f'[ $(grep -c "# ~BEGIN WT" /etc/hosts) == "0" ] && echo "\n# ~BEGIN WT\n\n# ~END WT" >> /etc/hosts'
        os.system(f'sudo /bin/bash -c \'{create}\'')

    def dump_config(self):
        plain_host = ''

        hosts = self.config.get_config('hosts')
        for host in hosts:
            plain_host += fr'{hosts[host]} {host}\n'

        sites = self.config.get_config('sites')
        for site in sites:
            site_config = sites.get(site)

            if site_config is None:
                continue

            aliases = site_config.get('alias')

            plain_host += f"127.0.0.1 {site}.test"

            if aliases:
                for alias in aliases:
                    plain_host += f' {alias}'

            plain_host += r'\n'

        cmd = r"sudo sed -i '\%# ~BEGIN WT%,\%# ~END WT%c # ~BEGIN WT\n'"
        cmd += f'"{plain_host}"' + "'# ~END WT' /etc/hosts"
        os.system(cmd)



import click
import os

tools = click.Group(name='tools')


@tools.command()
@click.argument('url')
def redirects(url):
    os.system("curl -sILk '{url}' | grep -E 'HTTP|Loc' | sed 's/Loc/ -> Loc/g'".format(url=url))
